package com.tzld.piaoquan.tencentad.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.tencentad.common.base.CommonResponse;
import com.tzld.piaoquan.tencentad.common.enums.AdVideoStatusEnum;
import com.tzld.piaoquan.tencentad.component.HttpPoolClient;
import com.tzld.piaoquan.tencentad.dao.mapper.AdCreativeMapper;
import com.tzld.piaoquan.tencentad.dao.mapper.AdGroupMapper;
import com.tzld.piaoquan.tencentad.model.po.*;
import com.tzld.piaoquan.tencentad.model.vo.AdCreativeVo;
import com.tzld.piaoquan.tencentad.model.vo.AdPutFlowParam;
import com.tzld.piaoquan.tencentad.model.vo.AdVideoVo;
import com.tzld.piaoquan.tencentad.service.AdAccountService;
import com.tzld.piaoquan.tencentad.service.AdCreativeService;
import com.tzld.piaoquan.tencentad.utils.DateUtils;
import com.tzld.piaoquan.tencentad.utils.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.net.URLDecoder;
import java.util.*;

@Slf4j
@Service
public class AdCreativeServiceImpl implements AdCreativeService {

    @Autowired
    private HttpPoolClient httpPoolClient;

    @Autowired
    private AdAccountService adAccountService;

    @Autowired
    private AdCreativeMapper adCreativeMapper;

    @Autowired
    private AdGroupMapper adGroupMapper;


    public boolean addAdCreative(AdVideo adVideo, List<Long> mediaIds, Long adgroupId, Integer accountId,
                                 String adGroupName, List<Long> adMuseIds, String description) {

        JSONObject param = new JSONObject();

        String page = insertPage(adVideo.getVideoId(), "");
        if (StringUtils.isEmpty(page)) {
            return false;
        }
        String rootSourceId = getRootSourceId(page);

        param.put("account_id", accountId);
        param.put("adgroup_id", adgroupId);
        param.put("delivery_mode", "DELIVERY_MODE_COMPONENT");
        param.put("dynamic_creative_type", "DYNAMIC_CREATIVE_TYPE_PROGRAM");
        param.put("dynamic_creative_name", rootSourceId);


        JSONObject creativeComponents = new JSONObject();
        JSONArray descriptions = new JSONArray();
        JSONObject description1 = new JSONObject();
        JSONObject descriptionValue = new JSONObject();
        descriptionValue.put("content", description);
        description1.put("value", descriptionValue);
        descriptions.add(description1);
        creativeComponents.put("description", descriptions);


        JSONArray images = new JSONArray();
        for (Long mediaId : mediaIds) {
            JSONObject image = new JSONObject();
            JSONObject imageValue = new JSONObject();
            imageValue.put("image_id", String.valueOf(mediaId));
            image.put("value", imageValue);
            images.add(image);
        }
        creativeComponents.put("image", images);


        JSONArray brands = new JSONArray();
        JSONObject brand1 = new JSONObject();
        JSONObject brand1Value = new JSONObject();
        brand1Value.put("brand_name", "一起成行");
        brand1Value.put("brand_image_id", "10372110611");
        brand1.put("value", brand1Value);
        brands.add(brand1);
        creativeComponents.put("brand", brands);

        JSONArray actionButtons = new JSONArray();
        JSONObject actionButton1 = new JSONObject();
        JSONObject actionButton1Value = new JSONObject();
        actionButton1Value.put("button_text", "了解更多");
        actionButton1.put("value", actionButton1Value);
        actionButtons.add(actionButton1);
        creativeComponents.put("action_button", actionButtons);

        JSONArray jumpInfos = getJumpInfos(page);
        JSONArray mainJumpInfo = getJumpInfos(page);

        creativeComponents.put("jump_info", jumpInfos);
        creativeComponents.put("main_jump_info", mainJumpInfo);
        param.put("creative_components", creativeComponents);

        String accessToken = adAccountService.getAccessToken(String.valueOf(accountId));
        Long timestamp = System.currentTimeMillis() / 1000;
        System.out.println(JSONObject.toJSONString(param));

        String url = String.format("https://api.e.qq.com/v3.0/dynamic_creatives/add?access_token=%s&timestamp=%s&nonce=%s",
                accessToken, timestamp, timestamp);
        try {
            String res = httpPoolClient.post(url, JSONObject.toJSONString(param));
            System.out.println(res);
            JSONObject jsonObject = JSONObject.parseObject(res);
            if (jsonObject != null && jsonObject.getInteger("code") == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                Long dynamicCreativeId = data.getLong("dynamic_creative_id");
                AdGroup adGroup = new AdGroup();
                adGroup.setAccountId(String.valueOf(accountId));
                adGroup.setAdgroupId(adgroupId);
                adGroup.setAdgroupName(adGroupName);
                adGroupMapper.insertSelective(adGroup);
                AdCreative adCreative = new AdCreative();
                adCreative.setAdGroupId(adGroup.getId());
                adCreative.setAdVideoId(adVideo.getId());
                adCreative.setDynamicCreativeId(dynamicCreativeId);
                adCreative.setMuseImageIds(JSONObject.toJSONString(adMuseIds));
                adCreativeMapper.insertSelective(adCreative);
                return true;
            }
        } catch (Exception e) {
            log.error("addAdCreative error", e);
        }
        return false;
    }

    @Override
    public CommonResponse<Page<AdCreativeVo>> getAdCreativeList(Integer pageNum, Integer pageSize) {
        Page<AdCreativeVo> page = new Page<>();
        page.setCurrentPage(pageNum);
        page.setPageSize(pageSize);
        AdCreativeExample example = new AdCreativeExample();
        example.setOrderByClause("create_time desc");
        example.setPage(page);
        List<AdCreative> adCreatives = adCreativeMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(adCreatives)) {
            return CommonResponse.success(page);
        }
        List<AdCreativeVo> adCreativeVos = new ArrayList<>();
        for (AdCreative adCreative : adCreatives) {
            AdCreativeVo adCreativeVo = new AdCreativeVo();
            BeanUtils.copyProperties(adCreative, adCreativeVo);
            adCreativeVo.setCreateTime(DateUtils.getDateString(adCreative.getCreateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adCreativeVo.setUpdateTime(DateUtils.getDateString(adCreative.getUpdateTime().getTime(), "yyyy-MM-dd HH:mm:ss"));
            adCreativeVos.add(adCreativeVo);
        }
        page.setObjs(adCreativeVos);
        return CommonResponse.success(page);
    }

    private JSONArray getJumpInfos(String page) {
        JSONArray jumpInfos = new JSONArray();
        JSONObject jumpInfo1 = new JSONObject();
        JSONObject jumpInfo1Value = new JSONObject();
        jumpInfo1Value.put("page_type", "PAGE_TYPE_WECHAT_MINI_PROGRAM");
        JSONObject wechatMiniProgramSpec = new JSONObject();
        wechatMiniProgramSpec.put("mini_program_id", "gh_f8e6629df9c6");
        wechatMiniProgramSpec.put("mini_program_path", page);
        JSONObject pageSpec = new JSONObject();
        pageSpec.put("wechat_mini_program_spec", wechatMiniProgramSpec);
        jumpInfo1Value.put("page_spec", pageSpec);
        jumpInfo1.put("value", jumpInfo1Value);
        jumpInfos.add(jumpInfo1);
        return jumpInfos;
    }

    private static String getRootSourceId(String page) {
        String decode = URLDecoder.decode(page);
        return decode.split("rootSourceId=")[1].split("&")[0];
    }

    private String insertPage(Long videoId, String remark) {
        AdPutFlowParam param = new AdPutFlowParam();
        param.setChannel("tencent");
        param.setPath("pages/category.html");
        param.setPutCarrierId("wxf7261ed54f2e450e");
        param.setPutScene("touliu");
        param.setPutStartDate(DateUtils.getDayDateString("yyyy-MM-dd"));
        param.setPutTypeOne("小程序");
        param.setRemark(remark);
        param.setVideoId(videoId);
        Map<String, String> requestParam = new HashMap<>();
        String jumpPage = "pages/user-videos?id=${videoId}&fromGzh=1&rootShareId=${uuid}&shareId=${uuid}&rootSourceId=[rootSourceId]"
                .replace("${videoId}", "" + videoId)
                .replace("${uuid}", "" + UUID.randomUUID());
        requestParam.put("jumpPage", jumpPage);
        param.setRequestParam(requestParam);
        try {
            String res = httpPoolClient.post("https://api.piaoquantv.com/ad/put/flow/add/tencent", JSONObject.toJSONString(param));
            JSONObject jsonObject = JSONObject.parseObject(res);
            JSONObject data = jsonObject.getJSONObject("data");
            return data.getString("url");
        } catch (Exception e) {
            log.error("insertPage error");
        }
        return null;
    }

}
