package com.tzld.piaoquan.tencentad.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.tencentad.component.HttpPoolClient;
import com.tzld.piaoquan.tencentad.service.TencentService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Slf4j
@Service
public class TencentServiceImpl implements TencentService {

    @Autowired
    private HttpPoolClient httpPoolClient;

    public String uploadImage(String accountId, String accessToken, String imageUrl) {
        String filePath = UUID.randomUUID() + ".jpg"; // 临时文件路径
        Path path = Paths.get(filePath);

        try {
            // 下载图片
            downloadImage(imageUrl, path);
            // 计算文件的 MD5 值
            String md5 = DigestUtils.md5Hex(Files.newInputStream(path));
            // 准备上传请求
            String uploadUrl = String.format("https://api.e.qq.com/v3.0/images/add?access_token=%s&timestamp=%s&nonce=%s",
                    accessToken, System.currentTimeMillis() / 1000, UUID.randomUUID().toString().replaceAll("-", ""));
            Map<String, String> param = new HashMap<>();
            param.put("account_id", accountId);
            param.put("upload_type", "UPLOAD_TYPE_FILE");
            param.put("signature", md5); // 使用计算出的 MD5 值
            // 上传文件
            File file = path.toFile();
            String res = httpPoolClient.post(uploadUrl, file, param);

            JSONObject jsonObject = JSONObject.parseObject(res);
            System.out.println(jsonObject);
            if (jsonObject != null && jsonObject.getInteger("code") == 0) {
                JSONObject data = jsonObject.getJSONObject("data");
                return data.getString("image_id");
            }
            return null;
        } catch (IOException e) {
            log.error("uploadImage error", e);
        } finally {
            // 删除临时文件
            try {
                Files.deleteIfExists(path);
            } catch (IOException e) {
                log.error("Failed to delete temporary file", e);
            }
        }
        return null;
    }

    private void downloadImage(String imageUrl, Path path) throws IOException {
        HttpURLConnection httpUrl = (HttpURLConnection) new URL(imageUrl).openConnection();
        httpUrl.connect();
        try (InputStream inputStream = httpUrl.getInputStream();
             OutputStream outputStream = Files.newOutputStream(path)) {
            byte[] buffer = new byte[4096];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }
        } finally {
            httpUrl.disconnect();
        }
    }


}
